@@ -4,13 +4,16 @@ use warnings;
 use Data::Dumper;
 
 use Getopt::Long qw(GetOptions);
+our @constants;
 BEGIN {
     my $err;
     eval '
         use Sereal::Encoder::Constants qw(:all);
+        @constants= @Sereal::Encoder::Constants::EXPORT_OK;
         1;
     ' or do { $err= $@; eval '
         use Sereal::Decoder::Constants qw(:all);
+        @constants= @Sereal::Decoder::Constants::EXPORT_OK;
         1;
     ' } or die "No encoder/decoder constants: $err\n$@";
 }
@@ -19,10 +22,9 @@ my $done;
 my $data;
 my $hlen;
 my $indent = "";
-my %const_names = map {$_ => eval "$_"} @Sereal::Constants::EXPORT_OK;
 
 sub parse_header {
-  $data =~ s/^(=srl)(.)// or die "invalid header: $data";
+  $data =~ s/^(=[s\xF3]rl)(.)// or die "invalid header: $data";
   $done .= $1 . $2;
   my $flags = $2;
   my $len = varint();
@@ -44,9 +46,13 @@ sub parse_header {
   else {
     print "Empty Header.\n";
   }
+
   my $encoding= ord($flags) & SRL_PROTOCOL_ENCODING_MASK;
 
-  if ($encoding == SRL_PROTOCOL_ENCODING_SNAPPY) {
+  printf "%i %i %i\n", $encoding, ord(SRL_PROTOCOL_ENCODING_MASK), ord($flags);
+  if ($encoding == SRL_PROTOCOL_ENCODING_RAW) {
+    print "Header says: Document body is uncompressed.\n";
+  } elsif ($encoding == SRL_PROTOCOL_ENCODING_SNAPPY) {
     print "Header says: Document body is Snappy-compressed.\n";
     require Compress::Snappy;
     my $out = Compress::Snappy::decompress($data);
@@ -57,7 +63,14 @@ sub parse_header {
     require Compress::Snappy;
     my $out = Compress::Snappy::decompress($data);
     $data = $out;
-  } elsif ($encoding) {
+  } elsif ($encoding == SRL_PROTOCOL_ENCODING_ZLIB) {
+    print "Header says: Document body is ZLIB-compressed.\n";
+    my $uncompressed_len = varint();
+    my $compressed_len = varint();
+    require Compress::Zlib;
+    my $out = Compress::Zlib::uncompress($data);
+    $data = $out;
+  } else {
     die "Invalid encoding '" . ($encoding >> SRL_PROTOCOL_VERSION_BITS) . "'";
   }
   $hlen= length($done);
@@ -102,6 +115,9 @@ sub parse_sv {
   if ($o == SRL_HDR_VARINT) {
     printf "VARINT: %u\n", varint();
   }
+  elsif ($o == SRL_HDR_ZIGZAG) {
+    printf "ZIGZAG: %d\n", zigzag();
+  }
   elsif (SRL_HDR_POS_LOW <= $o && $o <= SRL_HDR_POS_HIGH) {
     printf "POS: %u\n", $o;
   }
@@ -159,6 +175,9 @@ sub parse_sv {
     printf "HASH";
     parse_hv($ind);
   }
+  elsif ($o == SRL_HDR_CANONICAL_UNDEF) {
+    printf "CANONICAL_UNDEF\n";
+  }
   elsif ($o == SRL_HDR_UNDEF) {
     printf "UNDEF\n";
   }
@@ -213,7 +232,8 @@ sub parse_sv {
   }
   else {
     printf "<UNKNOWN>\n";
-    die "unsupported type: $o ($t): $const_names{$o}";
+    die sprintf "unsupported type: 0x%02x (%d) %s: %s", $o, $o,
+        Data::Dumper::qquote($t), Data::Dumper->new([$TAG_INFO_ARRAY[$o]])->Terse(1)->Dump();
   }
   return 0;
 }
@@ -266,6 +286,14 @@ sub varint {
   return $x;
 }
 
+BEGIN{
+my $_shift= length(pack"j",0) * 8 - 1;
+sub zigzag {
+    my $n= varint();
+    return ($n >> 1) ^ (-($n & 1));
+}
+}
+
 GetOptions(
   my $opt = {},
   'e|stderr',
@@ -276,8 +304,6 @@ if ($opt->{e}) {
   select(STDERR);
 }
 
-#print Dumper \%const_names; exit;
-
 local $/ = undef;
 $data = <STDIN>;
 
